/*******************************************************************************************************
File:		mystring.c
Author:		Norvin Eitzen
Project:	EiTL
Purpose:	Definitions of various string functions used with EiTL
********************************************************************************************************/

#include "../include/mystring.h"
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "../include/common.h"

/* For HTML stuff */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

/* From mystring.h
char *mid(char *string, long begin, long end);
int indexOf(char *string, char *tofind);
char *strRemove(char *string, char *string2);
char *strRemoveAll(char *string, char *string2);
char *left(char *string, long index);
char *right(char *string, long index);
char *strReplace(char *string, char *string2, char *string3);
char *strReplaceAll(char *string, char *string2, char *string3);
int occurencesOf(char *string, char *string2);
char *strToUpper(char *string); 
char *strToLower(char *string); 
int strIsAlpha(char *string);
int strIsNumber(char *string);
char *strTrim(char *string);

char *HTMLget_href(char *anchor);
int HTMLisFullHref(char *href);
int isIP(char *IP);
*/


/* =================================================================================
Function: 	mid
Paratmeters: 	string Pointer *string - Source String
	     	long begin - beginning index
	     	long end - Ending index
Returns: 	char * - String containing the substring of string from begin to end
Purpose: 	To retrieve a substring from a souce string bound by two indexes
Side Effects:	memory is allocated for the substring
Output:		None
====================================================================================*/
char *mid(char *string, long begin, long end) {
	long i,index=0;	char *done;
	if(begin>end) return "";
	if(end>strlen(string))return "";
	if((end == 0) && (begin == 0)) {
	 done = Allocate(2); done[0] = string[0]; done[1] = '\0';
	 return done;}
	done = Allocate((end-begin)+1);
	if(end==begin) sprintf(done, "%c", string[end]);
	else {
 	 for(i=begin;i<end;i++) done[index++]=string[i];
	 done[index]='\0';}
	return done;
}

/* ========================================================================================
Function:       left
Paratmeters:    string Pointer *string - Source String
                long index - number fo characters to return
Returns:        char * - String containing the substring of string from position 0 to index
Purpose:        To retrieve a substring from a souce string bound by position 0 to index
Side Effects:   memory is allocated for the substring (side effect of mid) 
Output:         None
==========================================================================================*/
char *left(char *string, long index) {	
	return(mid(string, 0, index));
}
/* ========================================================================================
Function:       right
Paratmeters:    string Pointer *string - Source String
                long index - number fo characters to return
Returns:        char * - String containing the substring of string from position index to length of string
Purpose:        To retrieve a substring from a souce string bound by position index to length of string
Side Effects:   memory is allocated for the substring (side effect of mid)
Output:         None
==========================================================================================*/
char *right(char *string, long index) {	
	return(mid(string, index, strlen(string)));
}

/* ========================================================================================
Function:       indexOf
Paratmeters:    string Pointer *string - Source String
                string pointer *toFind - substring to find
Returns:        integer - index of the beginning of the substring
		-1 if not found 
Purpose:        To find a substring within a srouce string
Side Effects:   None
Output:         None
==========================================================================================*/
int indexOf(char *string, char *toFind) {
	int i,j, counter,start;
	if(strstr(string, toFind)) {
	 j=0;start=-1;counter =0;
	 for(i=0;i<strlen(string);i++) {
	  if(string[i] == toFind[j]){
	   if(start == -1) start=i;
	   counter++;j++;}
	  else {counter=0;j=0;start=-1;}
	  if(counter==strlen(toFind)) return start;
         }	
	}else return -1;
}

/* ========================================================================================
Function:       strRemove
Paratmeters:    string Pointer *string - Source String
                string pointer *string2 - string to remove from source string
Returns:        char * - String with string2 removed (will be original string if not found)
Purpose:        To remove a substring from a souce string 
Side Effects:   memory is allocated for the substring (side effect of strReplace)
Output:         None
==========================================================================================*/
char *strRemove(char *string, char *string2){	
	return(strReplace(string, string2, ""));
}

/* ========================================================================================
Function:       strRemoveAll
Paratmeters:    string Pointer *string - Source String
                string pointer *string2 - string to remove from source string
Returns:        char * - String with string2 removed (will be original string if not found)
Purpose:        To remove all occurences of a substring from a souce string
Side Effects:   memory is allocated for the substring (side effect of strReplace)
Output:         None
==========================================================================================*/
char *strRemoveAll(char *string, char *string2) {
	return(strReplaceAll(string, string2, ""));
}

/* ========================================================================================
Function:       strReplace 
Paratmeters:    string Pointer *string - srouce string
		string Pointer *string2 - string to replace
		string pointer *string3 - string to replace with
Returns:        string pointer
Purpose:        to replace the first occurence of string2 with string3 in string
Side Effects:   memory allocation for returned string
Output:         none
==========================================================================================*/

char *strReplace(char *string, char *string2, char *string3) {
	/* Replace first occurence of string2 with string3 in string */

	int index;	
	char *done;
	done=Allocate(strlen(string) + strlen(string3) + 1);
	index=indexOf(string, string2);
	if(index != 0)
		sprintf(done, "%s%s%s", left(string, index),string3,mid(string, index+strlen(string2), strlen(string)));
	else sprintf(done, "%s%s", string3, mid(string,1,strlen(string)));;
	return done;

}
/* ========================================================================================
Function:       -
Paratmeters:    -
Returns:        -
Purpose:        -
Side Effects:   -
Output:         -
==========================================================================================*/

char *strReplaceAll(char *string, char *string2, char *string3) {
	char *done;
	done=Allocate(strlen(string)+strlen(string3)+1);
	strcpy(done, string);
	while(indexOf(done, string2) > -1)
		strcpy(done, strReplace(done, string2, string3));
	return done;
}
/* ========================================================================================
Function:       -
Paratmeters:    -
Returns:        -
Purpose:        -
Side Effects:   -
Output:         -
==========================================================================================*/

int occurencesOf(char *string, char *string2){
	char *done;
	int total=0;
	done=Allocate(strlen(string)+1);
	strcpy(done, string);
	while(indexOf(done, string2)>-1) {
		total++;
		strcpy(done,strRemove(done, string2));
	}
	free(done);
	return total;
}
/* ========================================================================================
Function:       -
Paratmeters:    -
Returns:        -
Purpose:        -
Side Effects:   -
Output:         -
==========================================================================================*/

char *strToUpper(char *string) {
	int i;char *ustr;
	ustr = (char *) malloc (strlen(string) * sizeof(char));
	for(i=0;i<strlen(string);i++)ustr[i] = toupper(string[i]);
	ustr[i]='\0';return(ustr);
}
/* ========================================================================================
Function:       -
Paratmeters:    -
Returns:        -
Purpose:        -
Side Effects:   -
Output:         -
==========================================================================================*/
 
char *strToLower(char *string) {
	int i;char *ustr;
	ustr = (char *) malloc (strlen(string) * sizeof(char));
	for(i=0;i<strlen(string);i++)ustr[i] = tolower(string[i]);
	ustr[i]='\0';return(ustr);
} 
/* ========================================================================================
Function:       -
Paratmeters:    -
Returns:        -
Purpose:        -
Side Effects:   -
Output:         -
==========================================================================================*/

char *strTrim(char *str) {
 int i,j,k; char *newstr;
 newstr = (char *) malloc (sizeof(char) * strlen(str));
 for(i=0;i<strlen(str);i++) if(!isspace(str[i])) { j=i;i=strlen(str);}
 for(i=strlen(str)-1;i>0;i--) if(!isspace(str[i])) { k=i+1;i=0;}
 strcpy(newstr, (char *) mid(str, j, k));
 newstr[k+1]='\0';
 return(newstr);
}

/* ========================================================================================
Function:       -
Paratmeters:    -
Returns:        -
Purpose:        -
Side Effects:   -
Output:         -
==========================================================================================*/

int strIsAlpha(char *string) {
 int i;
 for(i=0;i<strlen(string);i++) if(!isalpha(string[i])) return(-1);
 return(0);
}

/* ========================================================================================
Function:       -
Paratmeters:    -
Returns:        -
Purpose:        -
Side Effects:   -
Output:         -
==========================================================================================*/

int strIsNumber(char *string) {
 int i;
 for(i=0;i<strlen(string);i++) if(!isdigit(string[i])) return(-1);
 return(0);
}


/**********************************************************************************************************************/
/*	HTML Section												      */
/**********************************************************************************************************************/
/* ========================================================================================
Function:       -
Paratmeters:    -
Returns:        -
Purpose:        -
Side Effects:   -
Output:         -
==========================================================================================*/

char *HTMLget_href(char *anchor) {
        int index1,index2,index3;
        char *href;
        href = (char *) malloc (strlen(anchor) * sizeof(char));
         strcpy(anchor, (char *)strRemoveAll(anchor, " "));

        index1 = indexOf(strToUpper(anchor),"HREF=");
        index1 = index1 + 5;

        if(anchor[index1+1] == '\'') {
                index1++;
                /* Delimiter is a single quote */
                index2 = indexOf(mid(anchor, index1, strlen(anchor)), "'");

        }
        else if(anchor[index1+1] == '"') {
                /* Delimiter is a double quote */
                index1++;
                index2 = indexOf(mid(anchor, index1, strlen(anchor)), "\"");

        }
        else {
                /* Delimiter is a space or > */
                index2 = indexOf(mid(anchor, index1, strlen(anchor)), ">");
                index3 = indexOf(mid(anchor, index1, strlen(anchor)), " ");
                if((index3 < index2) && (index3 != -1) && (index2 != -1)) index2 = index3;
                else if((index2 == -1) && (index3 != -1)) index2 = index3;
        }
        strcpy(href, (char *)mid(anchor, index1, index2+index1));
        strcpy(href, (char *)strRemoveAll(href, "\""));
        return(href);
}

/* ========================================================================================
Function:       -
Paratmeters:    -
Returns:        -
Purpose:        -
Side Effects:   -
Output:         -
==========================================================================================*/

int HTMLisFullHref(char *href){

	/* Returns */
	/* 0 = good, full href */
	/* 1 = good, internal dir */
	/* -1 = bad, some other sort of href */
	
	/* Determines if the href has a domain with it or if it is an internal directory 04/06/2000*/
	
	/* eg.					*/ 
	/* www.yahoo.com = 0 			*/
	/* rf/j.html = 1			*/
	/* http://www.yahoo.com = 0 		*/
	/* mailto:jackass@blah.com = -1		*/
	/* javascript:jackass(); = -1		*/		
	/* 111.222.333.444			*/
	/* 111.222.333.444/r.html		*/
	/* ftp://ftp.freebsd.org		*/

	if(strcmp(href, "") == 0) return(-1);
	
	if(strcmp(mid(strToUpper(href),0,7), "MAILTO:")==0) return(-1);
	if(strcmp(mid(strToUpper(href),0,11), "JAVASCRIPT:")==0) return(-1);
	
	/* Check For leading HTTP */
	if(strcmp(mid(strToUpper(href),0,7), "HTTP://")==0) return(0);

	/* Check for leading FTP */
	if(strcmp(mid(strToUpper(href),0,6), "FTP://")==0) return(0);

	/* Check if there is a branch */
	if(indexOf(href,"/")>0) {
	 /* Branch Found */
	
	}
	else {
	 /* Could be IP, could be FQD */
	 
	 /* Check for IP */
	 if(isIP(href)>0) return(0);
	  
	} 	

		
	/* Incomplete 04/06/2000 */ 
	return(-1);
}


/* ========================================================================================
Function:       -
Paratmeters:    -
Returns:        -
Purpose:        -
Side Effects:   -
Output:         -
==========================================================================================*/

int isIP(char *IP) {
	void *addr_pt;int ret;
	ret=ascii2addr(AF_INET, IP, addr_pt);
	free(addr_pt);
	return ret;	
}
