#include <stdio.h>
#include <stdlib.h>
#include "../include/tcpcli.h"

char *grab_anchors(char *URL, char *tempbuf);
int write_to_links(char *URL, char *anchor);
char *get_href(char *URL, char *anchor);
int debug=0;

int main(int argc, char *argv[]) {
	if((argc < 2) && (argc > 4)) {
		printf("Usage: webbot <URL> <filename>\n");
		exit(1);
	}
	if((argc == 4) && (strcmp(argv[3],"1") == 0)) {
		debug=1;
		printf("Debug On\n");
	}
	if(gethttppage(argv[1], argv[2]) != 0) {
		printf("There was an error processing your request\n");
	}
	else {
		parsehtmlpage(argv[1], argv[2]);
		printf("Exiting Normally\n");
	}

	return(0);
}

/* ========================================================================================
Function:       parsehtmlpage
Paratmeters:    string - URL - A URL to retrieve
		string - filename - The file to dump the hypertext into 
Returns:        integer - 0=fine, 1=error 
Purpose:        To dump a hypertext page into a file  
Side Effects:   file with "filename" is either created or appeneded to.  
		Memory is allocated for string buffers
Output:         Error message upon file I/O error
==========================================================================================*/

int parsehtmlpage(char *URL, char *filename) {
	FILE *fp;
	char *leftoverbuf, *tempbuf;
	if(debug==1)printf("webbot.c:parsehtml(%s)\n", filename);
	leftoverbuf = NULL;
	tempbuf = NULL;
	
	if((fp = fopen(filename, "r")) == NULL) {
	 printf("Error opening %s\n", filename);
	 return(1); 
	}
	
	/* Wise idea to read file in as 1024 chunks, putting the entire file in memory could be dangerous :) */

	while(!feof(fp)) {

	 if(leftoverbuf == NULL) {
	  /* Was no leftovers from previous grab_anchors */
	  if(debug == 1) printf("webbot.c:parsehtml(%s):No Leftovers\n",filename);
	  tempbuf = (char *) malloc (1024 * sizeof(char));	
	  fread(tempbuf,1023,1,fp);
	  tempbuf[1024] = '\0';

	 }
	 else {
	  /* Use leftovers from previous grab_anchors.  tempbuf = leftoverbuf + new reading of 1024 */ 
	  if(debug == 1) printf("webbot.c:parsehtml(%s):Use leftovers [%s]\n",filename,leftoverbuf);  
  	  tempbuf = (char *) malloc ((1024 + strlen(leftoverbuf)) * sizeof(char));
	  strcpy(tempbuf, leftoverbuf);
	  free(leftoverbuf);
	  leftoverbuf = (char *) malloc (1024 * sizeof(char));
 	  fread(leftoverbuf, 1023, 1, fp);
	  leftoverbuf[1024] = '\0';
	  strcat(tempbuf, leftoverbuf);
	  free(leftoverbuf);
	  leftoverbuf = NULL;
	 }
 	
	 /* look for any anchors in the buffer */
	 tempbuf = grab_anchors(URL, tempbuf);
	 if(strlen(tempbuf) != 0) {
 	  /*left overs...damn! */
	  leftoverbuf = (char *) malloc (strlen(tempbuf) * sizeof(char));
	  strcpy(leftoverbuf, tempbuf);
	 }
	 
	}
	
	fclose(fp);
	return(0);
}
/* ========================================================================================
Function:       grab_anchors
Paratmeters:    string - URL - a URL
		string - tempbuf - a temporary string buffer
Returns:        string - left overs from incompleted hyperlinks  
Purpose:        To extract enchor tags from hypertext and dump them into a links file 
Side Effects:   Memory allocation which is freed aswell.
Output:         Error message while trying to write to links file
==========================================================================================*/

char *grab_anchors(char *URL, char *tempbuf) {
	int index1;
	int index2;
	int err;
	char *link;	
	int flag = 1;
	if(debug==1) printf("\nwebbot.c:grab_anchors(%s)\n", tempbuf);	
	while(flag != 0) {
		/* Grab as many anchors as possible.  Denoted as <a href ... </a> */
		index1 = 0;
		index2 = 0;
		index1 = indexOf(tempbuf, "<a href");
		index2 = indexOf(tempbuf, "</a>");
		flag = 0;
		if((index1 > 0) && (index2 > 0)) {
			link = (char *) malloc (((index2+4)-index1) * (sizeof(char))+1);
			link = (char *) mid(tempbuf, index1, index2+4);
			err = write_to_links(URL, link);
			if(err == 1) {
				fprintf(stderr, "System Failure...Exiting\n");
				exit(1);
			}
			flag = 1;
			strcpy(tempbuf, (char *)mid(tempbuf, index2+4, strlen(tempbuf) - (index2+4)));
		}
			
	}	
	free(link);
	if((index1 == 0) && (index2 > 0)) return(tempbuf);
	strcpy(tempbuf, "");
	tempbuf[0] = '\0';
	return(tempbuf);
}
/* ========================================================================================
Function:       write_to_links
Paratmeters:    string - URL - A URL
		string - anchor - The anchor tag to write 
Returns:        integer
		0 = success
		1 = failure 
Purpose:        To write an anchor tag to the links file 
Side Effects:   The links file is created or appended to 
Output:         If debug mode,error message when NULL argument passed 
==========================================================================================*/

int write_to_links(char *URL,char *anchor) {

	FILE *fp;
	char *href;

	printf("webbot.c:write_to_links(%s)\n", anchor);
	if(strcmp(anchor, "") ==0) {
		if(debug==1)printf("webbot.c:write_to_links(%s) - Rejeted:NULL argument\n", anchor);
		return(2);
	}
	if((fp = fopen("html/links.lst", "a+")) == NULL) {
		fprintf(stderr, "Error Opening html/links.lst\n");
		return(1);  
	}
	
	href = (char *) HTMLget_href(anchor);

	/* Determine if the href is a full address or an internal dir 04/06/2000*/   
	/* if(indexOf(strToUpper(href), "HTTP://") >0)
		fprintf(fp,"%s\n", href);
	else
		fprintf(fp, "http://%s/%s", URL,href);*/
	fprintf(fp, "%s\n", href);

	if(debug==1)printf("webbot.c:HTMLget_href(%s)=%s\n",anchor, HTMLget_href(anchor)); 
	fclose(fp);
	return(0);
}


